'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _ = require('./..');

var _2 = _interopRequireDefault(_);

var _layerModel = require('../layer-model');

var _layerModel2 = _interopRequireDefault(_layerModel);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function TestPlugin() {}
TestPlugin.prototype.setParams = function () {};
TestPlugin.prototype.setSQLParams = function () {};
TestPlugin.prototype.add = jest.fn();
TestPlugin.prototype.remove = jest.fn();
TestPlugin.prototype.setSource = jest.fn();
TestPlugin.prototype.setRender = jest.fn();
TestPlugin.prototype.setVisibility = jest.fn();
TestPlugin.prototype.setOpacity = jest.fn();
TestPlugin.prototype.setZIndex = jest.fn();
TestPlugin.prototype.setDecodeParams = jest.fn();
TestPlugin.prototype.getLayerByType = jest.fn().mockImplementation(function (type) {
  if (type !== 'success') {
    return function () {
      return {
        then: function then() {}
      };
    };
  }
  var mapLayer = { id: 'YET_ANOTHER_MAP_LAYER' };
  return function () {
    return Promise.resolve(mapLayer);
  };
});

describe('Core layer manager', function () {
  var MAP_INSTANCE = {};
  var layerManager = new _2.default(MAP_INSTANCE, TestPlugin);
  layerManager.requestCancel = jest.fn();

  beforeEach(function () {
    TestPlugin.prototype.add.mockClear();
    TestPlugin.prototype.remove.mockClear();
    TestPlugin.prototype.setSource.mockClear();
    TestPlugin.prototype.setRender.mockClear();
    TestPlugin.prototype.setVisibility.mockClear();
    TestPlugin.prototype.setOpacity.mockClear();
    TestPlugin.prototype.setZIndex.mockClear();
    TestPlugin.prototype.setDecodeParams.mockClear();
    TestPlugin.prototype.getLayerByType.mockClear();
    layerManager.requestCancel.mockClear();
  });

  it('adds layers to the map with default options', function () {
    var layer = { id: 'layer_0' };

    var layers = layerManager.add(layer);

    expect(layers).toBe(layerManager.layers);
    expect(layers.length).toBe(1);
    expect(layers[0]).toBeInstanceOf(_layerModel2.default);
    expect(layers[0]).toEqual({
      zIndex: 0,
      opacity: 1,
      id: layer.id,
      source: {},
      render: {},
      visibility: true,
      interactivity: null,
      changedAttributes: {}
    });
  });

  it('adds a layer with custom options', function () {
    var customOptions = {
      opacity: 0.5,
      visibility: false,
      zIndex: 22,
      interactivity: true,
      source: {},
      render: {},
      extra: 'wait what?' // is this expected behaviour?
    };
    var layer = { id: 'layer_1' };
    layerManager.add(layer, customOptions);

    expect(layerManager.layers[1]).toEqual(Object.assign({}, customOptions, {
      id: layer.id,
      changedAttributes: {}
    }));
  });

  it('returns new reference to layers using getLayers', function () {
    var newLayers = [].concat(_toConsumableArray(layerManager.layers));
    layerManager.layers = newLayers;
    expect(layerManager.getLayers()).toBe(newLayers);
  });

  it('removes a layer', function () {
    expect(layerManager.layers.length).toBe(2);
    layerManager.remove('layer_0');
    expect(layerManager.requestCancel).toHaveBeenCalled();
    expect(TestPlugin.prototype.remove).toHaveBeenCalled();
    expect(layerManager.layers.length).toBe(1);
    expect(layerManager.layers[0].id).toMatch('layer_1');
  });

  it('sets layer opacity using plugin', function () {
    var _layerManager$layers = _slicedToArray(layerManager.layers, 1),
        layer = _layerManager$layers[0];

    layerManager.setOpacity('layer_1', 0.8);
    expect(TestPlugin.prototype.setOpacity).toHaveBeenCalledWith(layer, 0.8);
  });

  it('sets layer visibility using plugin', function () {
    var _layerManager$layers2 = _slicedToArray(layerManager.layers, 1),
        layer = _layerManager$layers2[0];

    layerManager.setVisibility('layer_1', true);
    expect(TestPlugin.prototype.setVisibility).toHaveBeenCalledWith(layer, true);
  });

  it('sets layer z-index using plugin', function () {
    var _layerManager$layers3 = _slicedToArray(layerManager.layers, 1),
        layer = _layerManager$layers3[0];

    layerManager.setZIndex('layer_1', 11);
    expect(TestPlugin.prototype.setZIndex).toHaveBeenCalledWith(layer, 11);
  });

  it('ignores updates on missing layerModels and layerModels without mapLayer', function () {
    var changedProps = {
      opacity: 1,
      visibility: true,
      zIndex: 0,
      decodeParams: { stuff: [] }
    };

    layerManager.update('layer_0', changedProps);
    layerManager.update('layer_1', changedProps);
    expect(TestPlugin.prototype.setZIndex).not.toHaveBeenCalled();
    expect(TestPlugin.prototype.setOpacity).not.toHaveBeenCalled();
    expect(TestPlugin.prototype.setVisibility).not.toHaveBeenCalled();
    expect(TestPlugin.prototype.setDecodeParams).not.toHaveBeenCalled();
  });

  it('updates the layer and tracks changes in changedAttributes', function () {
    var changedProps = {
      opacity: 1,
      visibility: true,
      zIndex: 0
    };

    var originalLayer = Object.assign({}, layerManager.layers[0]);
    // we need a mapLayer in able to update a layer
    var mapLayer = { id: 'A_MAP_LAYER' };
    layerManager.layers[0].mapLayer = mapLayer;
    layerManager.update('layer_1', changedProps);

    expect(TestPlugin.prototype.setZIndex).toHaveBeenCalled();
    expect(TestPlugin.prototype.setOpacity).toHaveBeenCalled();

    // This seems to be a bug, it's not consistent.
    expect(TestPlugin.prototype.setVisibility).toHaveBeenCalled();
    expect(TestPlugin.prototype.setDecodeParams).not.toHaveBeenCalled();

    expect(layerManager.layers[0]).toEqual(Object.assign({}, originalLayer, changedProps, {
      mapLayer: mapLayer,
      // does it makes sense to save the data? Perhaps only the keys are needed?
      changedAttributes: changedProps
    }));
  });

  it('updates only the decodeParams', function () {
    var originalLayer = Object.assign({}, layerManager.layers[0]);
    var changedProps = {
      decodeParams: { key: 'someParams' }
    };
    layerManager.update('layer_1', changedProps);

    expect(TestPlugin.prototype.setZIndex).not.toHaveBeenCalled();
    expect(TestPlugin.prototype.setOpacity).not.toHaveBeenCalled();
    expect(TestPlugin.prototype.setVisibility).not.toHaveBeenCalled();
    expect(TestPlugin.prototype.setDecodeParams).toHaveBeenCalled();

    expect(layerManager.layers[0]).toEqual(Object.assign({}, originalLayer, changedProps, {
      // does it makes sense to save the data? Perhaps only the keys are needed?
      changedAttributes: changedProps
    }));
  });

  it('successfully resolves request with a mapLayer and adds it using plugin', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var newLayer, layer;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            newLayer = {
              id: 'layer_2',
              type: 'vector',
              source: {},
              render: {}
            };

            layerManager.add(newLayer);
            layer = layerManager.layers[1];

            layer.type = 'success';

            expect(layer.mapLayer).toBeUndefined();

            layerManager.requestLayer(layer, function () {});

            // we need to await the promise before doing assertions within a callback.
            _context.next = 8;
            return layerManager.promises[layer.id];

          case 8:

            expect(layerManager.requestCancel).toHaveBeenCalledWith(layer.id);
            expect(TestPlugin.prototype.add).toHaveBeenCalledWith(layer, layerManager.layers);
            expect(TestPlugin.prototype.setZIndex).toHaveBeenCalledWith(layer, layer.zIndex);
            expect(TestPlugin.prototype.setOpacity).toHaveBeenCalledWith(layer, layer.opacity);
            expect(TestPlugin.prototype.setVisibility).toHaveBeenCalledWith(layer, layer.visibility);
            expect(layer.mapLayer.id).toMatch('YET_ANOTHER_MAP_LAYER');

          case 14:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  })));
});