'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _isEmpty = require('lodash/isEmpty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _promise = require('./utils/promise');

var _promise2 = _interopRequireDefault(_promise);

var _layerModel = require('./layer-model');

var _layerModel2 = _interopRequireDefault(_layerModel);

var _providers = require('./providers');

var _providers2 = _interopRequireDefault(_providers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function checkPluginProperties(plugin) {
  if (plugin) {
    var requiredProperties = ['add', 'remove', 'setVisibility', 'setOpacity', 'setZIndex', 'setSource', 'setRender', 'setParams', 'setSQLParams', 'setDecodeParams', 'getLayerByType'];

    requiredProperties.forEach(function (property) {
      if (!plugin[property]) {
        console.error('The ' + property + ' function is required for layer manager plugins');
      }
    });

    var deprecatedProperties = ['getLayerByProvider'];
    deprecatedProperties.forEach(function (property) {
      if (plugin[property]) {
        console.warn('The ' + property + ' function is deprecated for layer manager plugins');
      }
    });
  }
}

var LayerManager = function () {
  function LayerManager(map, Plugin, Providers) {
    _classCallCheck(this, LayerManager);

    this.map = map;
    this.providers = Object.assign({}, _providers2.default, Providers);
    this.layers = [];
    this.promises = {};
    this.options = {
      getLayers: this.getLayers.bind(this)
    };

    this.plugin = new Plugin(this.map, this.options);
    checkPluginProperties(this.plugin);
  }

  /**
   * Add layers
   * @param {Array} layers
   * @param {Object} layerOptions
   */


  _createClass(LayerManager, [{
    key: 'add',
    value: function add(layer) {
      var layerOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
        opacity: 1,
        visibility: true,
        zIndex: 0,
        interactivity: null
      };
      var onAfterAdd = arguments[2];

      if (typeof layer === 'undefined') {
        console.error('layer is required');
        return this;
      }

      var layerModel = new _layerModel2.default(Object.assign({}, layer, layerOptions));
      var id = layerModel.id;


      if (this.layers.find(function (l) {
        return l.id === id;
      })) {
        return this.layers;
      }
      this.layers.push(layerModel);

      this.requestLayer(layerModel, onAfterAdd);

      return this.layers;
    }

    /**
     * Updating a specific layer
     * @param  {Object} layerModel
     */

  }, {
    key: 'update',
    value: function update(id, changedProps) {
      var layerModel = this.getLayerModel(id);
      if (!layerModel || !layerModel.mapLayer) return;

      layerModel.update(changedProps);

      var opacity = changedProps.opacity,
          visibility = changedProps.visibility,
          zIndex = changedProps.zIndex,
          source = changedProps.source,
          render = changedProps.render,
          params = changedProps.params,
          sqlParams = changedProps.sqlParams,
          decodeParams = changedProps.decodeParams;


      if (typeof opacity !== 'undefined') {
        this.plugin.setOpacity(layerModel, opacity);
      }

      if (typeof visibility !== 'undefined') {
        this.plugin.setVisibility(layerModel, visibility);
      }

      if (typeof zIndex !== 'undefined') {
        this.plugin.setZIndex(layerModel, zIndex);
      }

      if (!(0, _isEmpty2.default)(source)) {
        this.plugin.setSource(layerModel);
      }

      if (!(0, _isEmpty2.default)(render)) {
        this.plugin.setRender(layerModel);
      }

      if (!(0, _isEmpty2.default)(params)) {
        this.plugin.setParams(layerModel);
      }

      if (!(0, _isEmpty2.default)(sqlParams)) {
        this.plugin.setSqlParams(layerModel);
      }

      if (!(0, _isEmpty2.default)(decodeParams)) {
        this.plugin.setDecodeParams(layerModel);
      }
    }

    /**
     * Remove a layer giving a Layer ID
     * @param {String} id
     */

  }, {
    key: 'remove',
    value: function remove(id) {
      var layers = this.layers.slice(0);

      this.requestCancel(id);

      var layerModel = this.getLayerModel(id);

      if (layerModel) {
        this.plugin.remove(layerModel);
      }

      this.layers = layers.filter(function (l) {
        return l.id !== id;
      });
    }
  }, {
    key: 'getLayers',
    value: function getLayers() {
      return this.layers;
    }
  }, {
    key: 'getLayerModel',
    value: function getLayerModel(id) {
      return this.layers.find(function (l) {
        return l.id === id;
      });
    }

    /**
     * A namespace to set opacity on selected layer
     * @param {String} id
     * @param {Number} opacity
     */

  }, {
    key: 'setOpacity',
    value: function setOpacity(id, opacity) {
      var layerModel = this.getLayerModel(id);
      this.plugin.setOpacity(layerModel, opacity);
    }

    /**
     * A namespace to hide or show a selected layer
     * @param {String} id
     * @param {Boolean} visibility
     */

  }, {
    key: 'setVisibility',
    value: function setVisibility(id, visibility) {
      var layerModel = this.getLayerModel(id);
      this.plugin.setVisibility(layerModel, visibility);
    }

    /**
     * A namespace to set z-index on selected layer
     * @param {String} id
     * @param {Number} zIndex
     */

  }, {
    key: 'setZIndex',
    value: function setZIndex(id, zIndex) {
      var layerModel = this.getLayerModel(id);
      this.plugin.setZIndex(layerModel, zIndex);
    }
  }, {
    key: 'requestLayer',
    value: function requestLayer(layerModel, onAfterAdd) {
      var _this = this;

      var type = layerModel.type;

      var method = this.plugin.getLayerByType(type);

      if (!method) {
        this.promises[layerModel.id] = _promise2.default.reject(new Error(type + ' type is not yet supported.'));
        return false;
      }

      // Cancel previous/existing request
      this.requestCancel(layerModel.id);

      // every request method returns a promise that we store in the array
      // to control when all layers are fetched.
      this.promises[layerModel.id] = method.call(this, layerModel, this.providers).then(function (layer) {
        var canceled = _this.promises[layerModel.id]._canceled;

        if (!canceled) {
          layerModel.set('mapLayer', layer);

          _this.plugin.add(layerModel, _this.layers);
          _this.plugin.setZIndex(layerModel, layerModel.zIndex);
          _this.plugin.setOpacity(layerModel, layerModel.opacity);
          _this.plugin.setVisibility(layerModel, layerModel.visibility);

          _this.plugin.setRender(layerModel);

          onAfterAdd(layerModel);
        }
      });

      return this;
    }
  }, {
    key: 'requestCancel',
    value: function requestCancel(layerModelId) {
      // Cancel previous/existing request
      if (this.promises[layerModelId] && this.promises[layerModelId].cancel) {
        this.promises[layerModelId].cancel();
      }
    }
  }, {
    key: 'unmount',
    value: function unmount() {
      this.map = null;
      this.plugin.unmount();
    }
  }]);

  return LayerManager;
}();

exports.default = LayerManager;