'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _DEFAULT_TEXTURE_PARA;

var _constants = require('@luma.gl/constants');

var _constants2 = _interopRequireDefault(_constants);

var _core = require('@deck.gl/core');

var _luma = require('luma.gl');

var _images = require('@loaders.gl/images');

var _decodedLayerVertex = require('./decoded-layer-vertex');

var _decodedLayerVertex2 = _interopRequireDefault(_decodedLayerVertex);

var _decodedLayerFragment = require('./decoded-layer-fragment');

var _decodedLayerFragment2 = _interopRequireDefault(_decodedLayerFragment);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; } // Copyright (c) 2015 Uber Technologies, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

/* eslint-disable no-underscore-dangle,no-param-reassign,prefer-destructuring,class-methods-use-this */
/* global Image, HTMLCanvasElement */


var fp64LowPart = _luma.fp64.fp64LowPart;


var DEFAULT_TEXTURE_PARAMETERS = (_DEFAULT_TEXTURE_PARA = {}, _defineProperty(_DEFAULT_TEXTURE_PARA, _constants2.default.TEXTURE_MIN_FILTER, _constants2.default.NEAREST), _defineProperty(_DEFAULT_TEXTURE_PARA, _constants2.default.TEXTURE_MAG_FILTER, _constants2.default.NEAREST), _defineProperty(_DEFAULT_TEXTURE_PARA, _constants2.default.TEXTURE_WRAP_S, _constants2.default.CLAMP_TO_EDGE), _defineProperty(_DEFAULT_TEXTURE_PARA, _constants2.default.TEXTURE_WRAP_T, _constants2.default.CLAMP_TO_EDGE), _DEFAULT_TEXTURE_PARA);

var defaultProps = {
  image: null,
  bounds: { type: 'array', value: [1, 0, 0, 1], compare: true },
  fp64: false,

  desaturate: { type: 'number', min: 0, max: 1, value: 0 },
  // More context: because of the blending mode we're using for ground imagery,
  // alpha is not effective when blending the bitmap layers with the base map.
  // Instead we need to manually dim/blend rgb values with a background color.
  transparentColor: { type: 'color', value: [0, 0, 0, 0] },
  tintColor: { type: 'color', value: [255, 255, 255] }
};

/*
 * @class
 * @param {object} props
 * @param {number} props.transparentColor - color to interpret transparency to
 * @param {number} props.tintColor - color bias
 */

var DecodeLayer = function (_Layer) {
  _inherits(DecodeLayer, _Layer);

  function DecodeLayer() {
    _classCallCheck(this, DecodeLayer);

    return _possibleConstructorReturn(this, (DecodeLayer.__proto__ || Object.getPrototypeOf(DecodeLayer)).apply(this, arguments));
  }

  _createClass(DecodeLayer, [{
    key: 'getShaders',
    value: function getShaders() {
      var projectModule = 'project32';
      var fs = _decodedLayerFragment2.default.replace('{decodeParams}', Object.keys(this.props.decodeParams).map(function (p) {
        return 'uniform float ' + p + ';';
      }).join(' ')).replace('{decodeFunction}', this.props.decodeFunction || '');

      return { vs: _decodedLayerVertex2.default, fs: fs, modules: [projectModule, 'picking'] };
    }
  }, {
    key: 'initializeState',
    value: function initializeState() {
      var attributeManager = this.getAttributeManager();

      attributeManager.add({
        positions: {
          size: 3,
          update: this.calculatePositions,
          value: new Float32Array(12)
        },
        positions64xyLow: {
          size: 3,
          update: this.calculatePositions64xyLow,
          value: new Float32Array(12)
        }
      });

      this.setState({ numInstances: 4 }); // 4 corners
    }
  }, {
    key: 'updateState',
    value: function updateState(_ref) {
      var props = _ref.props,
          oldProps = _ref.oldProps;

      // setup model first
      if (props.fp64 !== oldProps.fp64) {
        var gl = this.context.gl;

        if (this.state.model) {
          this.state.model.delete();
        }
        this.setState({ model: this._getModel(gl) });
        this.getAttributeManager().invalidateAll();
      }

      if (props.image !== oldProps.image) {
        this.loadTexture(props.image);
      }

      var attributeManager = this.getAttributeManager();

      if (props.bounds !== oldProps.bounds) {
        this.setState({
          positions: this._getPositionsFromBounds(props.bounds)
        });
        attributeManager.invalidate('positions');
        attributeManager.invalidate('positions64xyLow');
      }
    }
  }, {
    key: 'finalizeState',
    value: function finalizeState() {
      _get(DecodeLayer.prototype.__proto__ || Object.getPrototypeOf(DecodeLayer.prototype), 'finalizeState', this).call(this);

      if (this.state.bitmapTexture) {
        this.state.bitmapTexture.delete();
      }
    }
  }, {
    key: '_getPositionsFromBounds',
    value: function _getPositionsFromBounds(bounds) {
      var positions = new Array(12);
      // bounds as [minX, minY, maxX, maxY]
      if (Number.isFinite(bounds[0])) {
        /*
          (minX0, maxY3) ---- (maxX2, maxY3)
                 |                  |
                 |                  |
                 |                  |
          (minX0, minY1) ---- (maxX2, minY1)
        */
        positions[0] = bounds[0];
        positions[1] = bounds[1];
        positions[2] = 0;

        positions[3] = bounds[0];
        positions[4] = bounds[3];
        positions[5] = 0;

        positions[6] = bounds[2];
        positions[7] = bounds[3];
        positions[8] = 0;

        positions[9] = bounds[2];
        positions[10] = bounds[1];
        positions[11] = 0;
      } else {
        // [[minX, minY], [minX, maxY], [maxX, maxY], [maxX, minY]]
        for (var i = 0; i < bounds.length; i++) {
          positions[i * 3 + 0] = bounds[i][0];
          positions[i * 3 + 1] = bounds[i][1];
          positions[i * 3 + 2] = bounds[i][2] || 0;
        }
      }

      return positions;
    }
  }, {
    key: '_getModel',
    value: function _getModel(gl) {
      if (!gl) {
        return null;
      }

      /*
        0,1 --- 1,1
         |       |
        0,0 --- 1,0
      */
      return new _luma.Model(gl, Object.assign({}, this.getShaders(), {
        id: this.props.id,
        shaderCache: this.context.shaderCache,
        geometry: new _luma.Geometry({
          drawMode: _constants2.default.TRIANGLE_FAN,
          vertexCount: 4,
          attributes: {
            texCoords: new Float32Array([0, 0, 0, 1, 1, 1, 1, 0])
          }
        }),
        isInstanced: false
      }));
    }
  }, {
    key: 'draw',
    value: function draw(_ref2) {
      var uniforms = _ref2.uniforms;
      var _state = this.state,
          bitmapTexture = _state.bitmapTexture,
          model = _state.model;
      var _props = this.props,
          desaturate = _props.desaturate,
          transparentColor = _props.transparentColor,
          tintColor = _props.tintColor,
          zoom = _props.zoom,
          decodeParams = _props.decodeParams,
          opacity = _props.opacity;

      // // TODO fix zFighting
      // Render the image

      if (bitmapTexture && model) {
        model.setUniforms(Object.assign({}, uniforms, Object.assign({
          bitmapTexture: bitmapTexture,
          desaturate: desaturate,
          transparentColor: transparentColor,
          tintColor: tintColor,
          zoom: zoom
        }, decodeParams, {
          opacity: opacity
        }))).draw();
      }
    }
  }, {
    key: 'loadTexture',
    value: function loadTexture(image) {
      var _this2 = this;

      if (typeof image === 'string') {
        image = (0, _images.loadImage)(image);
      }
      if (image instanceof Promise) {
        image.then(function (data) {
          return _this2.loadTexture(data);
        });
        return;
      }

      var gl = this.context.gl;


      if (this.state.bitmapTexture) {
        this.state.bitmapTexture.delete();
      }

      if (image instanceof _luma.Texture2D) {
        this.setState({ bitmapTexture: image });
      } else if (
      // browser object
      image instanceof Image || image instanceof HTMLCanvasElement) {
        this.setState({
          bitmapTexture: new _luma.Texture2D(gl, {
            data: image,
            parameters: DEFAULT_TEXTURE_PARAMETERS,
            mipmaps: false
          })
        });
      }
    }
  }, {
    key: 'calculatePositions',
    value: function calculatePositions(_ref3) {
      var value = _ref3.value;
      var positions = this.state.positions;

      value.set(positions);
    }
  }, {
    key: 'calculatePositions64xyLow',
    value: function calculatePositions64xyLow(attribute) {
      var isFP64 = this.use64bitPositions();
      attribute.constant = !isFP64;

      if (!isFP64) {
        attribute.value = new Float32Array(4);
        return;
      }

      var value = attribute.value;

      value.set(this.state.positions.map(fp64LowPart));
    }
  }]);

  return DecodeLayer;
}(_core.Layer);

exports.default = DecodeLayer;


DecodeLayer.layerName = 'DecodeLayer';
DecodeLayer.defaultProps = defaultProps;