'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _deck = require('deck.gl');

var _tileCache = require('./utils/tile-cache');

var _tileCache2 = _interopRequireDefault(_tileCache);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /* eslint-disable no-param-reassign,no-restricted-properties,class-methods-use-this,no-underscore-dangle */


var defaultProps = {
  renderSubLayers: { type: 'function', value: function value() {
      return null;
    } },
  getTileData: { type: 'function', value: function value() {
      return Promise.resolve(null);
    } },
  // TODO - change to onViewportLoad to align with Tile3DLayer
  onViewportLoaded: { type: 'function', optional: true, value: null },
  // eslint-disable-next-line
  onTileError: { type: 'function', value: function value(err) {
      return console.error(err);
    } },
  maxZoom: null,
  minZoom: 0,
  maxCacheSize: null
};

var TileLayer = function (_CompositeLayer) {
  _inherits(TileLayer, _CompositeLayer);

  function TileLayer() {
    _classCallCheck(this, TileLayer);

    return _possibleConstructorReturn(this, (TileLayer.__proto__ || Object.getPrototypeOf(TileLayer)).apply(this, arguments));
  }

  _createClass(TileLayer, [{
    key: 'initializeState',
    value: function initializeState() {
      this.state = {
        tiles: [],
        isLoaded: false
      };
    }
  }, {
    key: 'shouldUpdateState',
    value: function shouldUpdateState(_ref) {
      var changeFlags = _ref.changeFlags;

      return changeFlags.somethingChanged;
    }
  }, {
    key: 'updateState',
    value: function updateState(_ref2) {
      var props = _ref2.props,
          context = _ref2.context,
          changeFlags = _ref2.changeFlags;
      var tileCache = this.state.tileCache;

      if (!tileCache || changeFlags.updateTriggersChanged && (changeFlags.updateTriggersChanged.all || changeFlags.updateTriggersChanged.getTileData)) {
        var getTileData = props.getTileData,
            maxZoom = props.maxZoom,
            minZoom = props.minZoom,
            maxCacheSize = props.maxCacheSize;

        if (tileCache) {
          tileCache.finalize();
        }
        tileCache = new _tileCache2.default({
          getTileData: getTileData,
          maxSize: maxCacheSize,
          maxZoom: maxZoom,
          minZoom: minZoom,
          onTileLoad: this._onTileLoad.bind(this),
          onTileError: this._onTileError.bind(this)
        });

        this.setState({ tileCache: tileCache });
      } else if (changeFlags.updateTriggersChanged) {
        // if any updateTriggersChanged (other than getTileData), delete the layer
        this.state.tileCache.tiles.forEach(function (tile) {
          tile.layer = null;
        });
      }

      var viewport = context.viewport;

      if (changeFlags.viewportChanged && viewport.id !== 'DEFAULT-INITIAL-VIEWPORT') {
        var z = this.getLayerZoomLevel();
        tileCache.update(viewport);
        // The tiles that should be displayed at this zoom level
        var currTiles = tileCache.tiles.filter(function (tile) {
          return tile.z === z;
        });
        this.setState({ isLoaded: false, tiles: currTiles });
        this._onTileLoad();
      }
    }
  }, {
    key: '_onTileLoad',
    value: function _onTileLoad() {
      var onViewportLoaded = this.props.onViewportLoaded;

      var currTiles = this.state.tiles;
      var allCurrTilesLoaded = currTiles.every(function (tile) {
        return tile.isLoaded;
      });

      if (this.state.isLoaded !== allCurrTilesLoaded) {
        this.setState({ isLoaded: allCurrTilesLoaded });
        if (allCurrTilesLoaded && onViewportLoaded) {
          onViewportLoaded(currTiles.filter(function (tile) {
            return tile._data;
          }).map(function (tile) {
            return tile._data;
          }));
        }
      }
    }
  }, {
    key: '_onTileError',
    value: function _onTileError(error) {
      this.props.onTileError(error);
      // errorred tiles should not block rendering, are considered "loaded" with empty data
      this._onTileLoad();
    }
  }, {
    key: 'getPickingInfo',
    value: function getPickingInfo(_ref3) {
      var info = _ref3.info,
          sourceLayer = _ref3.sourceLayer;

      info.sourceLayer = sourceLayer;
      info.tile = sourceLayer.props.tile;
      return info;
    }
  }, {
    key: 'getLayerZoomLevel',
    value: function getLayerZoomLevel() {
      var z = Math.floor(this.context.viewport.zoom) + 1;
      var _props = this.props,
          maxZoom = _props.maxZoom,
          minZoom = _props.minZoom;

      if (Number.isFinite(maxZoom) && z > maxZoom) {
        return Math.floor(maxZoom);
      }

      if (Number.isFinite(minZoom) && z < minZoom) {
        return Math.ceil(minZoom);
      }
      return z;
    }
  }, {
    key: 'renderLayers',
    value: function renderLayers() {
      var _this2 = this;

      var _props2 = this.props,
          visible = _props2.visible,
          opacity = _props2.opacity,
          decodeParams = _props2.decodeParams,
          renderSubLayers = _props2.renderSubLayers;

      var z = this.getLayerZoomLevel();

      return this.state.tiles.map(function (tile) {
        // For a tile to be visible:
        // - parent layer must be visible
        // - tile must be visible in the current viewport
        // - if all tiles are loaded, only display the tiles from the current z level
        var isVisible = visible && tile.isVisible && (!_this2.state.isLoaded || tile.z === z);

        // cache the rendered layer in the tile
        if (!tile.layer) {
          // I have an error when I try this condition. I think is related to return the same tile.layer
          // https://github.com/uber/deck.gl/blob/c616dbfbcbdc9d8533b88ba8a142c5527764bbd8/modules/core/src/lib/layer-manager.js#L67-L70
          tile.layer = renderSubLayers(Object.assign({}, _this2.props, {
            id: _this2.id + '-' + tile.x + '-' + tile.y + '-' + tile.z,
            data: tile.data,
            visible: isVisible,
            opacity: opacity,
            tile: tile,
            zoom: z
          }));
        } else if (tile.layer.props.visible !== isVisible) {
          tile.layer = tile.layer.clone({
            visible: isVisible
          });
        } else if (tile.layer.props.opacity !== opacity) {
          tile.layer = tile.layer.clone({
            opacity: opacity
          });
        } else if (tile.layer.props.decodeParams) {
          tile.layer = tile.layer.clone({
            decodeParams: decodeParams
          });
        }
        return tile.layer;
      });
    }
  }]);

  return TileLayer;
}(_deck.CompositeLayer);

exports.default = TileLayer;


TileLayer.layerName = 'TileLayer';
TileLayer.defaultProps = defaultProps;