'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /* eslint-disable no-underscore-dangle */


var _sortBy = require('lodash/sortBy');

var _sortBy2 = _interopRequireDefault(_sortBy);

var _rasterLayerMapboxGl = require('./raster-layer-mapbox-gl');

var _rasterLayerMapboxGl2 = _interopRequireDefault(_rasterLayerMapboxGl);

var _vectorLayerMapboxGl = require('./vector-layer-mapbox-gl');

var _vectorLayerMapboxGl2 = _interopRequireDefault(_vectorLayerMapboxGl);

var _geojsonLayerMapboxGl = require('./geojson-layer-mapbox-gl');

var _geojsonLayerMapboxGl2 = _interopRequireDefault(_geojsonLayerMapboxGl);

var _videoLayerMapboxGl = require('./video-layer-mapbox-gl');

var _videoLayerMapboxGl2 = _interopRequireDefault(_videoLayerMapboxGl);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var PluginMapboxGL = function () {
  function PluginMapboxGL(map, options) {
    var _this = this;

    _classCallCheck(this, PluginMapboxGL);

    this.type = {
      raster: _rasterLayerMapboxGl2.default,
      vector: _vectorLayerMapboxGl2.default,
      geojson: _geojsonLayerMapboxGl2.default,
      video: _videoLayerMapboxGl2.default
    };

    this.map = map;
    this.options = options;

    // You can change mapStyles and all the layers will be repositioned
    this.map.on('style.load', function () {
      var layers = _this.getLayers();

      layers.forEach(function (layer) {
        return _this.add(layer);
      });
    });
  }

  _createClass(PluginMapboxGL, [{
    key: 'add',


    /**
     * Add a layer
     * @param {Object} layerModel
     */
    value: function add(layerModel) {
      var _this2 = this;

      var mapLayer = layerModel.mapLayer;

      var allLayers = this.getLayers();

      // add source if it has one
      if (this.map && mapLayer && mapLayer.source && mapLayer.id && !this.map.getSource(mapLayer.id)) {
        this.map.addSource(mapLayer.id, mapLayer.source);
      }

      // add layers
      if (mapLayer && mapLayer.layers) {
        mapLayer.layers.forEach(function (l) {
          var _l$metadata = l.metadata,
              metadata = _l$metadata === undefined ? {} : _l$metadata;

          var nextLayerId = metadata.position === 'top' ? null : _this2.getNextLayerId(layerModel);

          if (_this2.map && !_this2.map.getLayer(l.id)) {
            _this2.map.addLayer(l, nextLayerId);
          }

          allLayers.forEach(function () {
            _this2.setZIndex();
          });
        });
      }
    }

    /**
     * Remove a layer
     * @param {Object} layerModel
     */

  }, {
    key: 'remove',
    value: function remove() {
      var _this3 = this;

      var layerModel = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var mapLayer = layerModel.mapLayer;

      if (mapLayer && mapLayer.layers && this.map && this.map.style) {
        mapLayer.layers.forEach(function (l) {
          if (_this3.map.getLayer(l.id)) {
            _this3.map.removeLayer(l.id);
          }
        });
      }

      if (mapLayer && !!this.map && this.map.getSource(mapLayer.id)) {
        this.map.removeSource(mapLayer.id);
      }
    }

    /**
     * Get method by type
     * @param {String} type
     */

  }, {
    key: 'getLayerByType',
    value: function getLayerByType(type) {
      return this.type[type];
    }

    /**
     * Get all mapbox layers
     */

  }, {
    key: 'getLayersOnMap',
    value: function getLayersOnMap() {
      return this.map.getStyle().layers;
    }
  }, {
    key: 'getLayer',
    value: function getLayer(layerModel) {
      var mapLayer = layerModel.mapLayer;


      if (this.map) {
        return this.map.getSource(mapLayer.id);
      }

      return null;
    }

    /**
     * Get all layers passed to layer manager
     */

  }, {
    key: 'getLayers',
    value: function getLayers() {
      var getLayers = this.options.getLayers;

      var layers = getLayers();
      return (0, _sortBy2.default)(layers, function (l) {
        return l.decodeFunction;
      });
    }

    /**
     * Get the layer above the given z-index
     * @param {Object} layerModel
     */

  }, {
    key: 'getNextLayerId',
    value: function getNextLayerId(layerModel) {
      var zIndex = layerModel.zIndex;

      var allLayers = this.getLayers();
      var layersOnMap = this.getLayersOnMap();

      // find the top layer for placing data layers below
      var customLayer = layersOnMap && layersOnMap.length && layersOnMap.find(function (l) {
        return l.id.includes('custom-layers') || l.id.includes('label') || l.id.includes('place') || l.id.includes('poi');
      });

      // make sure layers are sorted by zIndex
      var sortedLayers = (0, _sortBy2.default)(allLayers, function (l) {
        return l.zIndex;
      });

      // get the layer with zIndex greater than current layer from all layers
      var nextLayer = sortedLayers.find(function (l) {
        return l.zIndex > zIndex && (!l.mapLayer || !l.mapLayer.layers || !l.mapLayer.layers[0] || !l.mapLayer.layers[0].metadata || !l.mapLayer.layers[0].metadata.position);
      });

      // if no layer above it then use the custom layer
      if (!nextLayer || !!nextLayer && !nextLayer.mapLayer) {
        return customLayer.id;
      }

      // get the first layer of the next layer's array
      var nextLayerMapLayers = nextLayer.mapLayer.layers;
      var nextLayerMapLayer = nextLayerMapLayers[0];

      // Filter layers with custom metadata position
      var id = nextLayerMapLayer.id;

      // if it has a layer above it, check if that layer has been added to the map and get its id

      var isNextLayerOnMap = !!layersOnMap.find(function (l) {
        return id === l.id;
      });

      // if next layer is on map return the id, else return the custom layer to add below
      return isNextLayerOnMap ? id : customLayer.id;
    }

    /**
     * A namespace to set z-index
     * @param {Object} layerModel
     */

  }, {
    key: 'setZIndex',
    value: function setZIndex() {
      var _this4 = this;

      var allLayers = this.getLayers();
      var layersOnMap = this.getLayersOnMap();

      // set zIndex for all layers currently on map
      layersOnMap.forEach(function (l) {
        var id = l.id,
            _l$metadata2 = l.metadata,
            metadata = _l$metadata2 === undefined ? {} : _l$metadata2;

        var layerModel = allLayers.find(function (ly) {
          return id.includes(ly.id);
        });

        if (layerModel) {
          var nextLayerId = metadata.position === 'top' ? null : _this4.getNextLayerId(layerModel);
          _this4.map.moveLayer(id, nextLayerId);
        }
      });

      // set for all decode layers that don't exist inside mapStyle()
      var decodeLayers = allLayers.filter(function (l) {
        return !!l.decodeFunction;
      });

      if (decodeLayers && this.map) {
        decodeLayers.forEach(function (layerModel) {
          var mapLayer = layerModel.mapLayer;


          if (mapLayer) {
            var layers = mapLayer.layers;

            var parentLayer = layers[0];
            var childLayer = layers[1];

            var parentLayerOnMap = layersOnMap.find(function (ly) {
              return ly.id === parentLayer.id;
            });
            var childLayerOnMap = _this4.map.getLayer(childLayer.id);

            if (parentLayerOnMap && childLayerOnMap) {
              _this4.map.moveLayer(childLayer.id, parentLayer.id);
            }
          }
        });
      }

      return true;
    }

    /**
     * A namespace to set opacity
     * @param {Object} layerModel
     * @param {Number} opacity
     */

  }, {
    key: 'setOpacity',
    value: function setOpacity(layerModel, opacity) {
      var _this5 = this;

      var PAINT_STYLE_NAMES = {
        symbol: ['icon', 'text'],
        circle: ['circle', 'circle-stroke']
      };

      var render = layerModel.render,
          mapLayer = layerModel.mapLayer,
          decodeFunction = layerModel.decodeFunction;
      var _render$layers = render.layers,
          layers = _render$layers === undefined ? [] : _render$layers;


      if (mapLayer.layers && !decodeFunction) {
        mapLayer.layers.forEach(function (l) {
          // Select the style to change depending on the type of layer
          var paintStyleNames = PAINT_STYLE_NAMES[l.type] || [l.type];

          // Select the paint property from the original layer

          var _ref = layers.find(function (v, i) {
            return (v.id || l.source + '-' + v.type + '-' + i) === l.id;
          }) || {},
              _ref$paint = _ref.paint,
              paint = _ref$paint === undefined ? {} : _ref$paint;

          // Loop each style name and check if there is an opacity in the original layer


          paintStyleNames.forEach(function (name) {
            var currentProperty = paint[name + '-opacity'];
            var paintOpacity = currentProperty !== undefined ? currentProperty : 1;
            _this5.map.setPaintProperty(l.id, name + '-opacity', paintOpacity * opacity * 0.99);
          });
        });
      }

      if (decodeFunction) {
        var layer = mapLayer.layers[1];

        if (layer && typeof layer.setProps === 'function') {
          layer.setProps({ opacity: opacity });
        }
      }
    }

    /**
     * A namespace to hide or show a selected layer
     * @param {Object} layerModel
     * @param {Boolean} visibility
     */

  }, {
    key: 'setVisibility',
    value: function setVisibility(layerModel, visibility) {
      var _this6 = this;

      var mapLayer = layerModel.mapLayer;


      if (mapLayer && mapLayer.layers && this.map && this.map.style) {
        mapLayer.layers.forEach(function (l) {
          _this6.map.setLayoutProperty(l.id, 'visibility', visibility ? 'visible' : 'none');
        });
      }
    }
  }, {
    key: 'setSource',
    value: function setSource(layerModel) {
      var id = layerModel.id,
          source = layerModel.source;
      var type = source.type,
          data = source.data;


      if (this.map && type === 'geojson' && !!data && typeof data !== 'string') {
        var src = this.map.getSource(id);
        src.setData(data);
      }
      return this;
    }
  }, {
    key: 'setRender',
    value: function setRender(layerModel) {
      var _this7 = this;

      var mapLayer = layerModel.mapLayer,
          render = layerModel.render;
      var _render$layers2 = render.layers,
          renderLayers = _render$layers2 === undefined ? [] : _render$layers2;


      if (!mapLayer || !renderLayers.length) {
        return this;
      }

      try {
        mapLayer.layers.forEach(function (layer, i) {
          var id = layer.id;

          var rl = renderLayers[i] || renderLayers[0] || {}; // take the style for each layer or use the first one for all of them
          var _rl$paint = rl.paint,
              paint = _rl$paint === undefined ? {} : _rl$paint,
              _rl$layout = rl.layout,
              layout = _rl$layout === undefined ? {} : _rl$layout,
              _rl$filter = rl.filter,
              filter = _rl$filter === undefined ? null : _rl$filter;


          _this7.map.setFilter(id, filter);

          Object.keys(paint).forEach(function (p) {
            // console.log(id, p, paint[p]);
            _this7.map.setPaintProperty(id, p, paint[p]);
          });

          Object.keys(layout).forEach(function (l) {
            _this7.map.setLayoutProperty(id, l, layout[l]);
          });
        });
      } catch (error) {
        console.error(error);
      }

      return this;
    }
  }, {
    key: 'setParams',
    value: function setParams() {
      return this;
    }
  }, {
    key: 'setSQLParams',
    value: function setSQLParams() {
      return this;
    }
  }, {
    key: 'setDecodeParams',
    value: function setDecodeParams(layerModel) {
      var mapLayer = layerModel.mapLayer,
          decodeParams = layerModel.decodeParams;


      if (!mapLayer) {
        return this;
      }

      var layer = mapLayer.layers[1];

      if (layer && typeof layer.setProps === 'function') {
        layer.setProps({ decodeParams: decodeParams });
      } else {
        console.error("Layer is not present. You defined decodeParams but maybe you didn't define a decodeFunction");
      }

      return this;
    }
  }, {
    key: 'unmount',
    value: function unmount() {
      this.map = null;
    }
  }]);

  return PluginMapboxGL;
}();

exports.default = PluginMapboxGL;