'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('../../utils/promise');

var _promise2 = _interopRequireDefault(_promise);

var _vectorStyleLayers = require('../../utils/vector-style-layers');

var _mapboxLayer = require('./custom-layers/mapbox-layer');

var _mapboxLayer2 = _interopRequireDefault(_mapboxLayer);

var _tileLayer = require('./custom-layers/tile-layer');

var _tileLayer2 = _interopRequireDefault(_tileLayer);

var _decodedLayer = require('./custom-layers/decoded-layer');

var _decodedLayer2 = _interopRequireDefault(_decodedLayer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var _getTileData = function _getTileData(_ref, url) {
  var x = _ref.x,
      y = _ref.y,
      z = _ref.z;

  var mapSource = url.replace('{z}', z).replace('{x}', x).replace('{y}', y);

  return fetch(mapSource).then(function (response) {
    return response.blob();
  }).then(function (response) {
    var _ref2 = response || {},
        type = _ref2.type;

    if (type !== 'application/xml' && type !== 'text/xml' && type !== 'text/html') {
      var src = URL.createObjectURL(response);
      var image = new Image();

      image.src = src;
      return image;
    }

    return false;
  });
};

var RasterLayer = function RasterLayer(layerModel, providers) {
  var _layerModel$source = layerModel.source,
      source = _layerModel$source === undefined ? {} : _layerModel$source,
      _layerModel$render = layerModel.render,
      render = _layerModel$render === undefined ? {} : _layerModel$render,
      decodeParams = layerModel.decodeParams,
      id = layerModel.id,
      opacity = layerModel.opacity,
      decodeFunction = layerModel.decodeFunction;


  var DEFAULT_RASTER_OPTIONS = {
    id: id + '-raster',
    type: 'raster',
    source: id
  };

  var provider = source.provider;
  var _render$layers = render.layers,
      layers = _render$layers === undefined ? [DEFAULT_RASTER_OPTIONS] : _render$layers;


  var layer = {};

  // if decoded layer use custom deck layer
  if (decodeFunction) {
    layer = {
      id: id,
      type: 'custom',
      layers: [{
        id: id + '-raster-decode-bg',
        type: 'background',
        paint: {
          'background-color': 'transparent'
        }
      }].concat(_toConsumableArray(source.tiles.map(function (t) {
        return new _mapboxLayer2.default({
          id: id + '-raster-decode',
          type: _tileLayer2.default,
          getTileData: function getTileData(e) {
            return _getTileData(e, t);
          },
          renderSubLayers: function renderSubLayers(_ref3) {
            var subLayerId = _ref3.id,
                data = _ref3.data,
                tile = _ref3.tile,
                visible = _ref3.visible,
                zoom = _ref3.zoom,
                decodeParamsSub = _ref3.decodeParams,
                decodeFunctionSub = _ref3.decodeFunction;

            if (data && data.src) {
              return new _decodedLayer2.default({
                id: subLayerId,
                image: data.src,
                bounds: tile.bbox,
                visible: visible,
                zoom: zoom,
                decodeParams: decodeParamsSub,
                decodeFunction: decodeFunctionSub,
                opacity: opacity
              });
            }
            return null;
          },
          minZoom: source.minzoom,
          maxZoom: source.maxzoom,
          opacity: layerModel.opacity,
          decodeParams: decodeParams,
          decodeFunction: decodeFunction
        });
      })))
    };
  } else {
    layer = {
      id: id,
      type: 'raster',
      source: Object.assign({
        type: 'raster',
        tileSize: 256
      }, source),
      layers: (0, _vectorStyleLayers.getVectorStyleLayers)(layers.map(function (l) {
        return Object.assign({}, DEFAULT_RASTER_OPTIONS, l);
      }), layerModel)
    };
  }

  if (provider) {
    var method = providers[provider.type];

    return new _promise2.default(function (resolve, reject) {
      if (!method) {
        reject(new Error(provider.type + ' provider is not supported. Try to add it to the providers method when you initialize layer-manager'));
      }

      method.call(undefined, layerModel, layer, resolve, reject);
    });
  }

  return new _promise2.default(function (resolve, reject) {
    if (layer) {
      resolve(layer);
    } else {
      reject(new Error('error in layer config'));
    }
  });
};

exports.default = RasterLayer;