'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _request = require('./utils/request');

exports.default = {
  carto: function carto(layerModel, layer, resolve, reject) {
    var interactivity = layerModel.interactivity,
        source = layerModel.source;
    var provider = source.provider;


    var layerTpl = JSON.stringify({
      version: '1.3.0',
      stat_tag: 'API',
      layers: provider.options.layers.map(function (l) {
        if (!!interactivity && interactivity.length) {
          return Object.assign({}, l, { options: Object.assign({}, l.options, { interactivity: interactivity }) });
        }
        return l;
      })
    });

    // https://carto.com/developers/auth-api/guides/how-to-send-API-Keys/
    var apiParams = Object.assign({
      stat_tag: 'API',
      config: encodeURIComponent(layerTpl)
    }, provider.options.api_key && { api_key: provider.options.api_key });
    var apiParamsString = Object.keys(apiParams).map(function (k) {
      return k + '=' + apiParams[k];
    }).join('&');
    var url = 'https://' + provider.options.account + '.carto.com/api/v1/map?' + apiParamsString;

    (0, _request.fetch)('get', url, {}, layerModel).then(function (response) {
      var ext = layerModel.type === 'vector' ? 'mvt' : 'png';
      var tileUrl = response.cdn_url.templates.https.url.replace('{s}', 'a') + '/' + provider.options.account + '/api/v1/map/' + response.layergroupid + '/{z}/{x}/{y}.' + ext;

      return resolve(Object.assign({}, layer, {
        source: Object.assign({}, layer.source, {
          tiles: [tileUrl]
        })
      }));
    }).catch(function (err) {
      return reject(err);
    });
  }
};