'use strict';

var _query = require('../query');

describe('Tests utils/query', function () {
  it('replace fn ignores typeof !== string', function () {
    var numberInput = 1245643;
    var result = (0, _query.replace)(numberInput);
    expect(result).toBe(numberInput);
  });

  it('substitution fn replaces all instances of the provided keys', function () {
    var input = "SELECT * from dummy where column = '{{firstColumn}}' and data = {{secondColumn}} and meta = {{secondColumn}}";
    var params = {
      firstColumn: 'MY_COLUMN',
      secondColumn: 1
    };
    expect((0, _query.substitution)(input, params)).toMatch('SELECT * from dummy where column = \'' + params.firstColumn + '\' and data = ' + params.secondColumn + ' and meta = ' + params.secondColumn);
  });

  it('concatenation fn adds optional parameters WHERE – AND', function () {
    var input = 'SELECT * from dummy {{where}} {{and}}';
    var sqlParams = {
      where: { column: 'MY_COLUMN' },
      and: { secondColumn: 'MY_SECOND' }
    };
    expect((0, _query.concatenation)(input, sqlParams)).toMatch('SELECT * from dummy WHERE column = \'' + sqlParams.where.column + '\' AND secondColumn = \'' + sqlParams.and.secondColumn + '\'');
  });

  it('concatenation fn adds multiple optional parameter WHERE – AND', function () {
    var input = 'SELECT * from dummy {{where}} {{and}} {{and2}}';
    var sqlParams = {
      where: { column: 'MY_COLUMN' },
      and: { secondColumn: 'MY_SECOND' },
      and2: { thirdColumn: 'MY_THIRD' }
    };
    expect((0, _query.concatenation)(input, sqlParams)).toMatch('SELECT * from dummy WHERE column = \'' + sqlParams.where.column + '\' AND secondColumn = \'' + sqlParams.and.secondColumn + '\' AND thirdColumn = \'' + sqlParams.and2.thirdColumn + '\'');
  });

  it('concatenation fn adds array and number parameters', function () {
    var input = 'SELECT * from dummy {{where}} {{and}} {{and2}}';
    var sqlParams = {
      where: { column: 1234 },
      and: { secondColumn: [1, 2, 3] }
    };
    expect((0, _query.concatenation)(input, sqlParams)).toMatch('SELECT * from dummy WHERE column = ' + sqlParams.where.column + ' AND secondColumn IN (' + sqlParams.and.secondColumn.join(', ') + ')');
  });
});