'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _vectorStyleLayers = require('../vector-style-layers');

describe('Returns mountable layers with id and opacity applied', function () {
  var LAYER_MODEL = { id: 1234, opacity: 0.5 };
  var VECTOR_LAYER = {
    paint: {
      'fill-color': '#ee9587',
      'fill-opacity': 0.7
    },
    'source-layer': 'layer0',
    type: 'fill'
  };

  it('returns false when no vectorLayers provided', function () {
    var noVectorLayers = (0, _vectorStyleLayers.getVectorStyleLayers)(null, LAYER_MODEL);
    var emptyVectorLayers = (0, _vectorStyleLayers.getVectorStyleLayers)([], LAYER_MODEL);

    expect(noVectorLayers).toBe(false);
    expect(emptyVectorLayers).toBe(false);
  });

  it('returns an id, layer source and layer model opacity applied up to 0.99', function () {
    var _getVectorStyleLayers = (0, _vectorStyleLayers.getVectorStyleLayers)([VECTOR_LAYER], LAYER_MODEL),
        _getVectorStyleLayers2 = _slicedToArray(_getVectorStyleLayers, 1),
        layer = _getVectorStyleLayers2[0];

    expect(layer).toEqual(Object.assign({}, VECTOR_LAYER, {
      source: LAYER_MODEL.id,
      id: LAYER_MODEL.id + '-' + VECTOR_LAYER.type + '-0',
      paint: Object.assign({}, VECTOR_LAYER.paint, {
        'fill-opacity': VECTOR_LAYER.paint['fill-opacity'] * LAYER_MODEL.opacity * 0.99
      })
    }));
  });

  it('sets null and undefined paint opacity props as 1', function () {
    var nullFillOpacityLayer = Object.assign({}, VECTOR_LAYER, {
      paint: Object.assign({}, VECTOR_LAYER.paint, {
        'fill-opacity': null
      })
    });

    var undefinedFillOpacityLayer = Object.assign({}, VECTOR_LAYER, {
      paint: Object.assign({}, VECTOR_LAYER.paint, {
        'fill-opacity': undefined
      })
    });

    var _getVectorStyleLayers3 = (0, _vectorStyleLayers.getVectorStyleLayers)([nullFillOpacityLayer, undefinedFillOpacityLayer], LAYER_MODEL),
        _getVectorStyleLayers4 = _slicedToArray(_getVectorStyleLayers3, 2),
        layer0 = _getVectorStyleLayers4[0],
        layer1 = _getVectorStyleLayers4[1];

    expect(layer0.paint['fill-opacity']).toBe(LAYER_MODEL.opacity * 0.99);
    expect(layer1.paint['fill-opacity']).toBe(LAYER_MODEL.opacity * 0.99);
  });

  it('removes paint props that are falsy', function () {
    var falsyPaintPropsLayer = Object.assign({}, VECTOR_LAYER, {
      paint: Object.assign({}, VECTOR_LAYER.paint, {
        'plain-false': false,
        'falsy-because-zero': 0, // is this the expected behaviour?
        'falsy-because-nan': NaN,
        'falsy-because-null': null,
        'falsy-because-undefined': undefined
      })
    });

    var _getVectorStyleLayers5 = (0, _vectorStyleLayers.getVectorStyleLayers)([falsyPaintPropsLayer], LAYER_MODEL),
        _getVectorStyleLayers6 = _slicedToArray(_getVectorStyleLayers5, 1),
        layer = _getVectorStyleLayers6[0];

    expect(layer.paint).toEqual(Object.assign({}, VECTOR_LAYER.paint, {
      'fill-opacity': VECTOR_LAYER.paint['fill-opacity'] * LAYER_MODEL.opacity * 0.99
    }));
  });
});