'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fetch = exports.post = exports.get = undefined;

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var headers = {
  'Content-Type': 'application/json'
};

var get = exports.get = function get(url) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return _axios2.default.get(url, Object.assign({
    headers: headers
  }, options));
};

var post = exports.post = function post(url, body) {
  return _axios2.default.post(url, body, { headers: headers });
};

var fetch = exports.fetch = function fetch(type, url, options, layerModel) {
  var layerRequest = layerModel.layerRequest;


  if (layerRequest) {
    layerRequest.cancel('Operation canceled by the user.');
  }

  var layerRequestSource = _axios.CancelToken.source();
  layerModel.set('layerRequest', layerRequestSource);

  var method = type === 'post' ? post : get;

  var newLayerRequest = method(url, Object.assign({}, options, {
    cancelToken: layerRequestSource.token
  })).then(function (res) {
    if (res.status > 400) {
      console.error(res);
      return false;
    }

    return res.data;
  });

  return newLayerRequest;
};

exports.default = { get: get, post: post, fetch: fetch };