'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getVectorStyleLayers = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _query = require('./query');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Parsed vector style layers that follow the mapbox style layer spec
 * and returns mountable layers with ids and opacity applied
 * @param {Array} vectorLayers
 * @param {Object} layerModel
 */
var getVectorStyleLayers = exports.getVectorStyleLayers = function getVectorStyleLayers(vectorLayers, layerModel) {
  var id = layerModel.id,
      params = layerModel.params,
      sqlParams = layerModel.sqlParams;

  if (vectorLayers && vectorLayers.length) {
    var vectorLayersParsed = JSON.parse((0, _query.replace)(JSON.stringify(vectorLayers), params, sqlParams));
    return vectorLayersParsed && vectorLayersParsed.map(function (vectorLayer, i) {
      var PAINT_STYLE_NAMES = {
        symbol: ['icon', 'text'],
        circle: ['circle', 'circle-stroke']
      };

      // Select the paint property from the original layer
      var _vectorLayer$paint = vectorLayer.paint,
          paint = _vectorLayer$paint === undefined ? {} : _vectorLayer$paint;

      // Select the style to change depending on the type of layer

      var opacityPaintNames = PAINT_STYLE_NAMES[vectorLayer.type] || [vectorLayer.type];

      var opacityPaintStyles = opacityPaintNames.reduce(function (obj, name) {
        var currentProperty = paint[name + '-opacity'];

        var paintOpacity = currentProperty !== undefined && currentProperty !== null ? currentProperty : 1;
        return Object.assign({}, obj, _defineProperty({}, name + '-opacity', paintOpacity * layerModel.opacity * 0.99));
      }, {});

      // if paint properties are null are passed it breaks interaction
      // on mapbox. We need to remove these
      var filteredPaintProperties = vectorLayer.paint && Object.entries(vectorLayer.paint).reduce(function (obj, _ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            key = _ref2[0],
            value = _ref2[1];

        return Object.assign({}, obj, !!value && _defineProperty({}, key, value));
      }, {});

      return Object.assign({
        // id: This will avoid having issues with any layer, but you should specify an id when you create it.
        // If you don't set an id in the definition, and you set a fill-opacity, it won't work
        id: id + '-' + vectorLayer.type + '-' + i
      }, vectorLayer, {
        source: id
      }, vectorLayer.paint && {
        paint: Object.assign({}, filteredPaintProperties, opacityPaintStyles)
      });
    });
  }

  return false;
};

exports.default = { getVectorStyleLayers: getVectorStyleLayers };